Index: xdata/xrddefault.c
===================================================================
--- xdata/xrddefault.c	(revision 1)
+++ xdata/xrddefault.c	(working copy)
@@ -98,6 +98,7 @@
 
 
 char *xrddefault_retention_file=NULL;
+char *xrddefault_sync_retention_file=NULL;
 char *xrddefault_temp_file=NULL;
 
 
@@ -117,6 +118,7 @@
 
 		log_debug_info(DEBUGL_RETENTIONDATA,2,"Error: Cannot open main configuration file '%s' for reading!\n",main_config_file);
 
+		my_free(xrddefault_sync_retention_file);
 		my_free(xrddefault_retention_file);
 		my_free(xrddefault_temp_file);
 
@@ -194,6 +196,9 @@
 	if(!strcmp(varname,"xrddefault_retention_file") || !strcmp(varname,"state_retention_file"))
 		xrddefault_retention_file=(char *)strdup(varvalue);
 
+	else if(!strcmp(varname,"sync_retention_file"))
+		xrddefault_sync_retention_file=(char *)strdup(varvalue);
+
 	/* temp file definition */
 	else if(!strcmp(varname,"temp_file"))
 		xrddefault_temp_file=(char *)strdup(varvalue);
@@ -230,6 +235,7 @@
 int xrddefault_cleanup_retention_data(char *config_file){
 
 	/* free memory */
+	my_free(xrddefault_sync_retention_file);
 	my_free(xrddefault_retention_file);
 	my_free(xrddefault_temp_file);
 
@@ -606,6 +612,17 @@
 /******************************************************************/
 
 int xrddefault_read_state_information(void){
+    return xrddefault_read_retention_file_information(xrddefault_retention_file,TRUE);
+}
+
+int xrddefault_sync_state_information(void){
+    int result=OK;
+    if(xrddefault_sync_retention_file!=NULL)
+        result=xrddefault_read_retention_file_information(xrddefault_sync_retention_file,FALSE);
+    return result;
+}
+
+int xrddefault_read_retention_file_information(char *retention_file, int overwrite_data){
 	char *input=NULL;
 	char *inputbuf=NULL;
 	char *temp_ptr=NULL;
@@ -658,12 +675,15 @@
 	struct timeval tv[2];
 	double runtime[2];
 	int found_directive=FALSE;
+	int retain_flag=TRUE;
+	time_t last_check=0L;
+    int add_object;
 
 
 	log_debug_info(DEBUGL_FUNCTIONS,0,"xrddefault_read_state_information() start\n");
 
 	/* make sure we have what we need */
-	if(xrddefault_retention_file==NULL){
+	if(retention_file==NULL){
 
 #ifdef NSCORE
 		logit(NSLOG_RUNTIME_ERROR,TRUE,"Error: We don't have a filename for retention data!\n");
@@ -676,7 +696,7 @@
 		gettimeofday(&tv[0],NULL);
 
 	/* open the retention file for reading */
-	if((thefile=mmap_fopen(xrddefault_retention_file))==NULL)
+	if((thefile=mmap_fopen(retention_file))==NULL)
 		return ERROR;
 
 	/* what attributes should be masked out? */
@@ -749,7 +769,7 @@
 
 			case XRDDEFAULT_HOSTSTATUS_DATA:
 
-				if(temp_host!=NULL){
+				if(temp_host!=NULL && retain_flag){
 
 					/* adjust modified attributes if necessary */
 					if(temp_host->retain_nonstatus_information==FALSE)
@@ -808,6 +828,7 @@
 				/* reset vars */
 				was_flapping=FALSE;
 				allow_flapstart_notification=TRUE;
+				retain_flag=TRUE;
 
 				my_free(host_name);
 				host_name=NULL;
@@ -816,7 +837,7 @@
 
 			case XRDDEFAULT_SERVICESTATUS_DATA:
 
-				if(temp_service!=NULL){
+				if(temp_service!=NULL && retain_flag){
 
 					/* adjust modified attributes if necessary */
 					if(temp_service->retain_nonstatus_information==FALSE)
@@ -879,6 +900,7 @@
 				/* reset vars */
 				was_flapping=FALSE;
 				allow_flapstart_notification=TRUE;
+				retain_flag=TRUE;
 
 				my_free(host_name);
 				my_free(service_description);
@@ -915,35 +937,56 @@
 			case XRDDEFAULT_HOSTCOMMENT_DATA:
 			case XRDDEFAULT_SERVICECOMMENT_DATA:
 
-				/* add the comment */
-				add_comment((data_type==XRDDEFAULT_HOSTCOMMENT_DATA)?HOST_COMMENT:SERVICE_COMMENT,entry_type,host_name,service_description,entry_time,author,comment_data,comment_id,persistent,expires,expire_time,source);
+                add_object=TRUE;
 
-				/* delete the comment if necessary */
-				/* it seems a bit backwards to add and then immediately delete the comment, but its necessary to track comment deletions in the event broker */
-				remove_comment=FALSE;
-				/* host no longer exists */
-				if((temp_host=find_host(host_name))==NULL)
-					remove_comment=TRUE;
-				/* service no longer exists */
-				else if(data_type==XRDDEFAULT_SERVICECOMMENT_DATA && (temp_service=find_service(host_name,service_description))==NULL)
-					remove_comment=TRUE;
-				/* acknowledgement comments get deleted if they're not persistent and the original problem is no longer acknowledged */
-				else if(entry_type==ACKNOWLEDGEMENT_COMMENT){
-					ack=FALSE;
-					if(data_type==XRDDEFAULT_HOSTCOMMENT_DATA)
-						ack=temp_host->problem_has_been_acknowledged;
-					else
-						ack=temp_service->problem_has_been_acknowledged;
-					if(ack==FALSE && persistent==FALSE)
-						remove_comment=TRUE;
-					}
-				/* non-persistent comments don't last past restarts UNLESS they're acks (see above) */
-				else if(persistent==FALSE)
-					remove_comment=TRUE;
-				
-				if(remove_comment==TRUE)
-					delete_comment((data_type==XRDDEFAULT_HOSTCOMMENT_DATA)?HOST_COMMENT:SERVICE_COMMENT,comment_id);
+                if(overwrite_data==FALSE) {
+                    /* search for comment. If exists, then drop. If doesn't exist, then flow through but need to get a comment_id */
+                    if(find_comment_by_similar_content((data_type==XRDDEFAULT_HOSTCOMMENT_DATA)?HOST_COMMENT:SERVICE_COMMENT,host_name,(data_type==XRDDEFAULT_HOSTCOMMENT_DATA?NULL:service_description),author,comment_data)!=NULL)
+                        add_object=FALSE;
+                    else {
+                        /* Get next available comment_id */
+                        while(find_comment(next_comment_id,ANY_COMMENT)!=NULL)
+                            next_comment_id++;
+                        comment_id=next_comment_id;
+                    }
+                }
 
+                if(add_object==TRUE) {
+                    /* add the comment */
+                    add_comment((data_type==XRDDEFAULT_HOSTCOMMENT_DATA)?HOST_COMMENT:SERVICE_COMMENT,entry_type,host_name,service_description,entry_time,author,comment_data,comment_id,persistent,expires,expire_time,source);
+
+                    /* delete the comment if necessary */
+                    /* it seems a bit backwards to add and then immediately delete the comment, but its necessary to track comment deletions in the event broker */
+                    remove_comment=FALSE;
+                    /* host no longer exists */
+                    if((temp_host=find_host(host_name))==NULL)
+                        remove_comment=TRUE;
+                    /* service no longer exists */
+                    else if(data_type==XRDDEFAULT_SERVICECOMMENT_DATA && (temp_service=find_service(host_name,service_description))==NULL)
+                        remove_comment=TRUE;
+                    /* acknowledgement comments get deleted if they're not persistent and the original problem is no longer acknowledged */
+                    else if(entry_type==ACKNOWLEDGEMENT_COMMENT){
+                        ack=FALSE;
+                        if(data_type==XRDDEFAULT_HOSTCOMMENT_DATA)
+                            ack=temp_host->problem_has_been_acknowledged;
+                        else
+                            ack=temp_service->problem_has_been_acknowledged;
+                        if(ack==FALSE && persistent==FALSE)
+                            remove_comment=TRUE;
+                        }
+                    /* non-persistent comments don't last past restarts UNLESS they're acks (see above) */
+                    else if(persistent==FALSE)
+                        remove_comment=TRUE;
+                    
+                    if(remove_comment==TRUE)
+                        delete_comment((data_type==XRDDEFAULT_HOSTCOMMENT_DATA)?HOST_COMMENT:SERVICE_COMMENT,comment_id);
+
+                    /* Reset any temporary pointers */
+                    temp_host=NULL;
+                    temp_service=NULL;
+
+                }
+
 				/* free temp memory */
 				my_free(host_name);
 				my_free(service_description);
@@ -964,15 +1007,31 @@
 			case XRDDEFAULT_HOSTDOWNTIME_DATA:
 			case XRDDEFAULT_SERVICEDOWNTIME_DATA:
 
-				/* add the downtime */
-				if(data_type==XRDDEFAULT_HOSTDOWNTIME_DATA)
-					add_host_downtime(host_name,entry_time,author,comment_data,start_time,end_time,fixed,triggered_by,duration,downtime_id);
-				else
-					add_service_downtime(host_name,service_description,entry_time,author,comment_data,start_time,end_time,fixed,triggered_by,duration,downtime_id);
+                add_object=TRUE;
 
-				/* must register the downtime with Nagios so it can schedule it, add comments, etc. */
-				register_downtime((data_type==XRDDEFAULT_HOSTDOWNTIME_DATA)?HOST_DOWNTIME:SERVICE_DOWNTIME,downtime_id);
+                if(overwrite_data==FALSE) {
+                    /* search for downtime. If exists, then drop. If doesn't exist, then flow through but need to get a downtime id */
+                    if(find_downtime_by_similar_content(ANY_DOWNTIME,host_name,(data_type==XRDDEFAULT_HOSTDOWNTIME_DATA?NULL:service_description),author,comment_data,start_time,end_time,fixed,duration)!=NULL)
+                        add_object=FALSE;
+                    else {
+                        /* Get next available downtime id */
+                        while(find_downtime(ANY_DOWNTIME,next_downtime_id)!=NULL)
+                            next_downtime_id++;
+                        downtime_id=next_downtime_id;
+                    }
+                }
 
+                if (add_object==TRUE) {
+                    /* add the downtime */
+                    if(data_type==XRDDEFAULT_HOSTDOWNTIME_DATA)
+                        add_host_downtime(host_name,entry_time,author,comment_data,start_time,end_time,fixed,triggered_by,duration,downtime_id);
+                    else
+                        add_service_downtime(host_name,service_description,entry_time,author,comment_data,start_time,end_time,fixed,triggered_by,duration,downtime_id);
+
+                    /* must register the downtime with Nagios so it can schedule it, add comments, etc. */
+                    register_downtime((data_type==XRDDEFAULT_HOSTDOWNTIME_DATA)?HOST_DOWNTIME:SERVICE_DOWNTIME,downtime_id);
+                }
+
 				/* free temp memory */
 				my_free(host_name);
 				my_free(service_description);
@@ -1175,7 +1234,7 @@
 						/* break out */
 						break;
 						}
-					if(temp_host->retain_status_information==TRUE){
+					if(retain_flag && temp_host->retain_status_information==TRUE){
 						if(!strcmp(var,"has_been_checked"))
 							temp_host->has_been_checked=(atoi(val)>0)?TRUE:FALSE;
 						else if(!strcmp(var,"check_execution_time"))
@@ -1202,8 +1261,13 @@
 							my_free(temp_host->perf_data);
 							temp_host->perf_data=(char *)strdup(val);
 					                }
-						else if(!strcmp(var,"last_check"))
-							temp_host->last_check=strtoul(val,NULL,10);
+						else if(!strcmp(var,"last_check")) {
+							last_check=strtoul(val,NULL,10);
+							if (overwrite_data==FALSE && last_check<temp_host->last_check)
+								retain_flag=FALSE;
+							else
+								temp_host->last_check=last_check;
+							}
 						else if(!strcmp(var,"next_check")){
 							if(use_retained_scheduling_info==TRUE && scheduling_info_is_ok==TRUE)
 								temp_host->next_check=strtoul(val,NULL,10);
@@ -1263,7 +1327,7 @@
 						else
 							found_directive=FALSE;
 					        }
-					if(temp_host->retain_nonstatus_information==TRUE){
+					if(retain_flag && temp_host->retain_nonstatus_information==TRUE){
 						/* null-op speeds up logic */
 						if(found_directive==TRUE);
 
@@ -1446,7 +1510,7 @@
 						/* mask out attributes we don't want to retain */
 						temp_service->modified_attributes&=~service_attribute_mask;
 						}
-					if(temp_service->retain_status_information==TRUE){
+					if(retain_flag && temp_service->retain_status_information==TRUE){
 						if(!strcmp(var,"has_been_checked"))
 							temp_service->has_been_checked=(atoi(val)>0)?TRUE:FALSE;
 						else if(!strcmp(var,"check_execution_time"))
@@ -1497,8 +1561,13 @@
 							my_free(temp_service->perf_data);
 							temp_service->perf_data=(char *)strdup(val);
 					                }
-						else if(!strcmp(var,"last_check"))
-							temp_service->last_check=strtoul(val,NULL,10);
+						else if(!strcmp(var,"last_check")){
+							last_check=strtoul(val,NULL,10);
+							if (overwrite_data==FALSE && last_check<temp_service->last_check)
+								retain_flag=FALSE;
+							else
+								temp_service->last_check=last_check;
+					                }
 						else if(!strcmp(var,"next_check")){
 							if(use_retained_scheduling_info==TRUE && scheduling_info_is_ok==TRUE)
 								temp_service->next_check=strtoul(val,NULL,10);
@@ -1538,7 +1607,7 @@
 						else
 							found_directive=FALSE;
 					        }
-					if(temp_service->retain_nonstatus_information==TRUE){
+					if(retain_flag && temp_service->retain_nonstatus_information==TRUE){
 						/* null-op speeds up logic */
 						if(found_directive==TRUE);
 
@@ -1861,6 +1930,10 @@
 	my_free(inputbuf);
 	mmap_fclose(thefile);
 
+    /* If this is a sync file, remove the file */
+    if(overwrite_data==FALSE)
+        unlink(retention_file);
+
 	if(test_scheduling==TRUE)
 		gettimeofday(&tv[1],NULL);
 
@@ -1877,5 +1950,7 @@
 		printf("\n\n");
 		}
 
+	log_debug_info(DEBUGL_FUNCTIONS,0,"xrddefault_read_state_information() end\n");
+
 	return OK;
         }
Index: xdata/xrddefault.h
===================================================================
--- xdata/xrddefault.h	(revision 1)
+++ xdata/xrddefault.h	(working copy)
@@ -43,5 +43,6 @@
 int xrddefault_grab_config_directives(char *);
 int xrddefault_save_state_information(void);        /* saves all host and service state information */
 int xrddefault_read_state_information(void);        /* reads in initial host and service state information */
+int xrddefault_sync_state_information(void);        /* syncs in host and service state information via sync file */
 
 #endif
Index: html/docs/configmain.html
===================================================================
--- html/docs/configmain.html	(revision 1)
+++ html/docs/configmain.html	(working copy)
@@ -1696,6 +1696,56 @@
 
 
 
+<a name="sync_retention_file"></a>
+
+<table border="0" width="100%" class="Default">
+
+<tr>
+
+<td bgcolor="#cbcbcb"><strong>Sync Retention File</strong></td>
+
+</tr>
+
+</table>
+
+<br>
+
+
+
+
+
+<table border="0" class="Default">
+
+<tr>
+
+<td>Format:</td>
+
+<td><strong>sync_retention_file=&lt;file_name&gt;</strong></td>
+
+</tr>
+
+<tr>
+
+<td>Example:</td>
+
+<td><font color="red"><strong>sync_retention_file=/usr/local/nagios/var/sync.dat</strong></font></td>
+
+</tr>
+
+</table>
+
+
+
+<p>
+
+This is an advanced option which works like the state_retention_file so that you can load a subset of retention information such as status, acknowledgements, downtimes and comments (you will have to create the contents of this file outside of Nagios). When Nagios is restarted, it reads the information in the sync_retention_file and will update the host or service specified <em>if the last_update time in the sync file is newer</em> than in the state_retention_file, otherwise information will be discarded for that host or service. After the sync file has been read, it will be removed. To disable, comment out the option.
+
+</p>
+
+
+
+
+
 <a name="retention_update_interval"></a>
 
 <table border="0" width="100%" class="Default">
Index: include/sretention.h
===================================================================
--- include/sretention.h	(revision 1)
+++ include/sretention.h	(working copy)
@@ -30,6 +30,7 @@
 int cleanup_retention_data(char *);
 int save_state_information(int);                 /* saves all host and state information */
 int read_initial_state_information(void);        /* reads in initial host and state information */
+int sync_state_information(void);                /* syncs hosts and state information from sync file */
 
 #ifdef __cplusplus
   }
Index: include/comments.h
===================================================================
--- include/comments.h	(revision 1)
+++ include/comments.h	(working copy)
@@ -46,6 +46,7 @@
 
 #define HOST_COMMENT			1
 #define SERVICE_COMMENT			2
+#define ANY_COMMENT             3
 
 
 /****************************** ENTRY TYPES ********************************/
@@ -105,6 +106,7 @@
 comment *find_comment(unsigned long,int);                             /* finds a specific comment */
 comment *find_service_comment(unsigned long);                         /* finds a specific service comment */
 comment *find_host_comment(unsigned long);                            /* finds a specific host comment */
+comment *find_comment_by_similar_content(int,char *,char *,char *,char *); /* finds a comment based on content - used for distributed nagios instances */
 
 comment *get_first_comment_by_host(char *);
 comment *get_next_comment_by_host(char *,comment *);
Index: include/downtime.h
===================================================================
--- include/downtime.h	(revision 1)
+++ include/downtime.h	(working copy)
@@ -98,6 +98,7 @@
 scheduled_downtime *find_downtime(int,unsigned long);
 scheduled_downtime *find_host_downtime(unsigned long);
 scheduled_downtime *find_service_downtime(unsigned long);
+scheduled_downtime *find_downtime_by_similar_content(int,char *,char *,char *,char *,time_t,time_t,int,unsigned long);
 
 void free_downtime_data(void);                                       /* frees memory allocated to scheduled downtime list */
 
Index: include/common.h
===================================================================
--- include/common.h	(revision 1)
+++ include/common.h	(working copy)
@@ -299,6 +299,8 @@
 #define CMD_CHANGE_CONTACT_MODHATTR                     168
 #define CMD_CHANGE_CONTACT_MODSATTR                     169
 
+#define CMD_SYNC_STATE_INFORMATION			            170
+
 /* custom command introduced in Nagios 3.x */
 #define CMD_CUSTOM_COMMAND                              999
 
Index: sample-config/nagios.cfg.in
===================================================================
--- sample-config/nagios.cfg.in	(revision 1)
+++ sample-config/nagios.cfg.in	(working copy)
@@ -640,7 +640,22 @@
 state_retention_file=@localstatedir@/retention.dat
 
 
+# SYNC FILE
+# This is an advanced facility to pass a subset of retention information into Nagios on a running system.
+# This is similar to the state retention file with the following difference:
+# - if the last_check value is less than the current last_check, then the state information is ignored (this must be
+#   specified immediately after the object identifiers)
+# - downtimes and comments are not identified by an id number, but by other "similar characteristics". This is required to
+#   work in a distributed Nagios environment
+#    * downtimes: hostname, servicename (if appropriate), author, comment, start_time, end_time, fixed, duration
+#    * comments:  hostname, servicename, author, comment
+# If this variable is set, then on Nagios startup, the sync file will be read after the retention file has been
+# processed. If the file is read successfully, it will be removed
+# If the file does not exist, no error will appear
+# There is also an API that will force a read of the sync file
+#sync_retention_file=@localstatedir@/sync.dat
 
+
 # RETENTION DATA UPDATE INTERVAL
 # This setting determines how often (in minutes) that Nagios
 # will automatically save retention data during normal operation.
Index: t-tap/test_nagios_config.c
===================================================================
--- t-tap/test_nagios_config.c	(revision 1)
+++ t-tap/test_nagios_config.c	(working copy)
@@ -240,7 +240,6 @@
 extern timeperiod      *timeperiod_list;
 extern serviceescalation *serviceescalation_list;
 extern host 		*host_list;
-extern char	*xrddefault_retention_file;
 
 notification    *notification_list;
 
@@ -318,11 +317,13 @@
 	struct tm *tm;
 	time_t now;
 	char datestring[256];
-	host *temp_host=NULL;
+	host *temp_host=NULL, *temp_host2=NULL;
 	hostgroup *temp_hostgroup=NULL;
 	hostsmember *temp_member=NULL;
+    service *temp_service=NULL;
+    scheduled_downtime *temp_downtime=NULL;
 
-	plan_tests(7);
+	plan_tests(48);
 
 	/* reset program variables */
 	reset_variables();
@@ -356,14 +357,102 @@
 		//printf("host pointer=%d\n", temp_member->host_ptr);
 	}
 
+    /* Initial configuration */
+	initialize_retention_data(config_file);
+	
 	temp_host=find_host("host1");
 	ok( temp_host->current_state==0, "State is assumed OK on initial load" );
+	ok( temp_host->last_check==0, "No last_check time" );
+	ok( temp_host->problem_has_been_acknowledged==0, "No problem_has_been_acknowledged" );
 
-	xrddefault_retention_file=strdup("smallconfig/retention.dat");
-	ok( xrddefault_read_state_information() == OK, "Reading retention data" );
+	temp_host2=find_host("hostveryrecent");
+	ok( temp_host2->current_state==0, "State is assumed OK on initial load" );
+	ok( temp_host2->last_check==0, "No last_check time" );
 
+    temp_service=find_service("host1", "Dummy service");
+    ok( temp_service->current_state==0, "OK" );
+    ok( temp_service->last_check==0, "No last_check" );
+    ok( temp_service->problem_has_been_acknowledged==0, "No problem_has_been_acknowledged" );
+
+
+    /* With retained data */
+	ok( read_initial_state_information() == OK, "Reading retention data" );
+
 	ok( temp_host->current_state==1, "State changed due to retention file settings");
+	ok( temp_host->last_check==1111111111, "Last check read from retention file");
+	ok( temp_host->problem_has_been_acknowledged==1, "problem_has_been_acknowledged=1" );
 
+	ok( temp_host2->current_state==1, "State changed due to retention file settings");
+	ok( temp_host2->last_check==1264456433, "Last check read from retention file");
+
+    ok( temp_service->current_state==0, "No changes because not in retention data" );
+    ok( temp_service->last_check==0, "No last_check" );
+    ok( temp_service->problem_has_been_acknowledged==0, "No problem_has_been_acknowledged" );
+
+    temp_downtime = find_downtime(ANY_DOWNTIME,19);
+    ok( temp_downtime!=NULL, "Found downtime");
+    ok( strcmp(temp_downtime->comment,"Downtime set way into future")==0, "With right comment" );
+
+
+    /* With sync_retention data */
+    ok( system("cp smallconfig/sync.dat.in smallconfig/sync.dat")==0, "Copied in sync.dat");
+        
+	ok( sync_state_information() == OK, "Reading sync_retention data" );
+	
+	ok( temp_host->current_state==0, "State changed due to sync.dat file" );
+	ok( temp_host->last_check==1264496382, "Last check changed due to sync.dat" );
+	ok( temp_host->problem_has_been_acknowledged==0, "problem_has_been_acknowledged=0" );
+
+	ok( temp_host2->current_state==1, "State NOT changed due to sync.dat file" );
+	ok( temp_host2->last_check==1264456433, "Last check NOT changed due to later last_check time" );
+
+    ok( temp_service->current_state==1, "Changes due to sync.dat file" );
+    ok( temp_service->last_check==1264511929, "Latest last_check" );
+    ok( temp_service->problem_has_been_acknowledged==1, "problem_has_been_acknowledged" );
+
+
+
+    temp_service = find_service("host1", "Dummy service2");
+    ok( temp_service!=NULL, "Found dummy service2");
+    ok( strcmp(temp_service->plugin_output, "CRITICAL - some snow is falling!")==0, "Got right output" );
+
+
+
+
+    temp_downtime = find_downtime(ANY_DOWNTIME,184);
+    ok( temp_downtime!=NULL, "Found sync'd downtime");
+    ok( temp_downtime->type==SERVICE_DOWNTIME, "Found service downtime" );
+    ok( strcmp(temp_downtime->service_description,"Dummy service")==0, "Got right service name" );
+    ok( strcmp(temp_downtime->comment,"Synchronised downtime")==0, "With right comment" );
+
+    //printf("downtime id=%d\ntype=%d\nhost=%s\nservice=%s\nuser=%s\ncomment=%s\n",temp_downtime->downtime_id,temp_downtime->type,temp_downtime->host_name,temp_downtime->service_description,temp_downtime->author,temp_downtime->comment);
+
+    temp_downtime = find_downtime_by_similar_content(ANY_DOWNTIME,"host1","Dummy service","admin","Synchronised downtime",(time_t)2034567991,(time_t)2034567995,1,9);
+    ok( temp_downtime!=NULL, "Got a downtime when searching for similar service");
+    ok( temp_downtime->downtime_id==184, "Got downtime searching for similar item" );
+
+
+    temp_downtime = find_downtime_by_similar_content(ANY_DOWNTIME,"host1",NULL,"admin","Synchronised downtime",(time_t)2034567991,(time_t)2034567995,1,9);
+    ok( temp_downtime!=NULL, "Got a downtime when searching for a similar host downtime");
+    ok( temp_downtime->downtime_id==185, "Got downtime searching for similar item" );
+    ok( temp_downtime->type==HOST_DOWNTIME, "Is a host" );
+
+
+    temp_downtime = find_downtime_by_similar_content(ANY_DOWNTIME,"host1","Dummy service2","admin","Synchronised downtime",(time_t)2034567991,(time_t)2034567995,1,9);
+    ok( temp_downtime!=NULL, "Check a 2nd time as there is a host downtime squashed in between these two service downtimes");
+    ok( temp_downtime->downtime_id==186, "Got right item" );
+
+
+
+
+    temp_downtime = find_downtime_by_similar_content(SERVICE_DOWNTIME,"host1",NULL,"admin","Synchronised downtime",(time_t)2034567991,(time_t)2034567995,1,9);
+    ok( temp_downtime==NULL, "Cannot find a service if NULL specified for service name");
+
+
+
+    temp_downtime = find_downtime_by_similar_content(ANY_DOWNTIME,"nonexistent",NULL,"admin","Synchronised downtime",(time_t)2034567991,(time_t)2034567995,1,9);
+    ok( temp_downtime==NULL, "No result from search" );
+
 	cleanup();
 
 	my_free(config_file);
Index: t-tap/smallconfig/minimal.cfg
===================================================================
--- t-tap/smallconfig/minimal.cfg	(revision 1)
+++ t-tap/smallconfig/minimal.cfg	(working copy)
@@ -10,6 +10,18 @@
 	hostgroups hostgroup1
 }
 
+define host {
+	host_name hostveryrecent
+	alias	hostveryrecent test
+	address	192.168.1.1
+	max_check_attempts 2
+	check_period	none
+	contacts	nagiosadmin
+	notification_interval 60
+	notification_period none
+	hostgroups hostgroup1
+}
+
 define hostgroup {
 	hostgroup_name hostgroup1
 }
@@ -33,6 +45,19 @@
 
 define service {
 	host_name	host1
+	service_description Dummy service2
+	check_command	check_me
+	max_check_attempts	3
+	check_interval	32
+	retry_interval	1
+	check_period	none
+	notification_interval	60
+	notification_period	none
+	contacts	nagiosadmin
+}
+
+define service {
+	host_name	host1
 	service_description	Uses important check command
 	check_command	check_me!with some parameters
 	max_check_attempts	5
Index: t-tap/smallconfig/sync.dat.in
===================================================================
--- t-tap/smallconfig/sync.dat.in	(revision 0)
+++ t-tap/smallconfig/sync.dat.in	(revision 21)
@@ -0,0 +1,285 @@
+host {
+host_name=host1
+last_check=1264496382
+current_state=0
+plugin_output=
+problem_has_been_acknowledged=0
+}
+service {
+host_name=host1
+service_description=Check Loadavg
+last_check=1264511923
+current_state=2
+plugin_output=CHECK_NRPE: Socket timeout after 10 seconds.
+problem_has_been_acknowledged=0
+}
+service {
+host_name=host1
+service_description=/
+last_check=1264511819
+current_state=2
+plugin_output=CHECK_NRPE: Socket timeout after 10 seconds.
+problem_has_been_acknowledged=0
+}
+service {
+host_name=host1
+service_description=Dummy service
+last_check=1264511929
+current_state=1
+plugin_output=WARNING - host1: rta 109.076ms, lost 20%
+problem_has_been_acknowledged=1
+}
+servicecomment {
+host_name=host1
+service_description=Dummy service
+entry_type=2
+source=0
+persistent=0
+entry_time=1264511283
+expires=0
+expire_time=0
+author=(Nagios Process)
+comment_data=Inserted between two other services because I think Nagios is not resetting a variable correctly
+}
+service {
+host_name=host1
+service_description=Dummy service2
+last_check=1264511929
+current_state=2
+plugin_output=CRITICAL - some snow is falling!
+problem_has_been_acknowledged=0
+}
+service {
+host_name=host1
+service_description=SSH
+last_check=1264511825
+current_state=2
+plugin_output=CRITICAL - Socket timeout after 10 seconds
+problem_has_been_acknowledged=0
+}
+service {
+host_name=host1
+service_description=Check Memory
+last_check=1264512021
+current_state=2
+plugin_output=CHECK_NRPE: Socket timeout after 10 seconds.
+problem_has_been_acknowledged=0
+}
+service {
+host_name=host1
+service_description=VNC
+last_check=1264512027
+current_state=2
+plugin_output=CRITICAL - Socket timeout after 10 seconds
+problem_has_been_acknowledged=0
+}
+host {
+host_name=hostveryrecent
+last_check=1111111111
+current_state=0
+plugin_output=Host is up!
+problem_has_been_acknowledged=0
+}
+service {
+host_name=hostveryrecent
+service_description=snmptrap renotify
+last_check=1264511343
+current_state=0
+plugin_output=(No output returned from plugin)
+problem_has_been_acknowledged=0
+}
+service {
+host_name=hostveryrecent
+service_description=Passive set_stale
+last_check=1264511343
+current_state=1
+plugin_output=**Disk failure** on *sd1* - see [internal wiki](http://nagios.org) < 10
+problem_has_been_acknowledged=0
+}
+service {
+host_name=hostveryrecent
+service_description=Passive renotify
+last_check=1264511343
+current_state=0
+plugin_output=(No output returned from plugin)
+problem_has_been_acknowledged=0
+}
+service {
+host_name=hostveryrecent
+service_description=Passive no freshness
+last_check=1260870977
+current_state=0
+plugin_output=Service assumed OK - no results received
+problem_has_been_acknowledged=0
+}
+service {
+host_name=hostveryrecent
+service_description=Test exceptions
+last_check=1263904158
+current_state=0
+plugin_output=(No output returned from plugin)
+problem_has_been_acknowledged=0
+}
+service {
+host_name=hostveryrecent
+service_description=Coldstart
+last_check=1264511343
+current_state=1
+plugin_output=**Disk failure** on *sd1* - see [internal wiki](http://nagios.org) < 10
+problem_has_been_acknowledged=0
+}
+service {
+host_name=hostveryrecent
+service_description=snmptrap no freshness
+last_check=1260870977
+current_state=0
+plugin_output=Service assumed OK - no results received
+problem_has_been_acknowledged=0
+}
+service {
+host_name=hostveryrecent
+service_description=snmptrap set_stale
+last_check=1264511343
+current_state=1
+plugin_output=**Disk failure** on *sd1* - see [internal wiki](http://nagios.org) < 10
+problem_has_been_acknowledged=0
+}
+service {
+host_name=hostveryrecent
+service_description=Another exception
+last_check=1264511343
+current_state=0
+plugin_output=(No output returned from plugin)
+problem_has_been_acknowledged=0
+}
+host {
+host_name=slavehost
+last_check=1261006749
+current_state=1
+plugin_output=
+problem_has_been_acknowledged=0
+}
+service {
+host_name=slavehost
+service_description=Check Loadavg
+last_check=1264511934
+current_state=2
+plugin_output=CHECK_NRPE: Socket timeout after 10 seconds.
+problem_has_been_acknowledged=0
+}
+service {
+host_name=slavehost
+service_description=/
+last_check=1264511831
+current_state=2
+plugin_output=CHECK_NRPE: Socket timeout after 10 seconds.
+problem_has_been_acknowledged=0
+}
+service {
+host_name=slavehost
+service_description=Mysql Query
+last_check=1264511836
+current_state=3
+plugin_output=Usage: check_mysql_query -q SQL_query [-w warn] [-c crit] [-H host] [-P port] [-s socket]
+problem_has_been_acknowledged=0
+}
+service {
+host_name=slavehost
+service_description=SSH
+last_check=1264511940
+current_state=2
+plugin_output=CRITICAL - Socket timeout after 10 seconds
+problem_has_been_acknowledged=0
+}
+service {
+host_name=slavehost
+service_description=Check Memory
+last_check=1264512033
+current_state=2
+plugin_output=CHECK_NRPE: Socket timeout after 10 seconds.
+problem_has_been_acknowledged=0
+}
+service {
+host_name=slavehost
+service_description=VNC
+last_check=1264512038
+current_state=2
+plugin_output=CRITICAL - Socket timeout after 10 seconds
+problem_has_been_acknowledged=0
+}
+servicecomment {
+host_name=slavehost
+service_description=VNC
+entry_type=2
+source=0
+persistent=0
+entry_time=1264511283
+expires=0
+expire_time=0
+author=(Nagios Process)
+comment_data=This service has been scheduled for fixed downtime from 31-12-2010 10:19:24 to 31-12-2010 12:19:24.  Notifications for the service will not be sent out during that time period.
+}
+hostcomment {
+host_name=slavehost
+entry_type=2
+source=0
+persistent=0
+entry_time=1264511283
+expires=0
+expire_time=0
+author=(Nagios Process)
+comment_data=This host has been scheduled for fixed downtime from 31-12-2010 10:19:24 to 31-12-2010 12:19:24.  Notifications for the host will not be sent out during that time period.
+}
+hostdowntime {
+host_name=cisco3
+entry_time=1262254781
+start_time=1293790764
+end_time=1293797964
+fixed=1
+duration=7200
+author=admin
+comment=Host group 'Leaf': test future downtime
+}
+servicedowntime {
+host_name=host1
+service_description=Dummy service
+entry_time=1264627948
+start_time=2034567991
+end_time=2034567995
+fixed=1
+duration=9
+author=admin
+comment=Synchronised downtime
+}
+hostdowntime {
+host_name=host1
+entry_time=1264627948
+start_time=2034567991
+end_time=2034567995
+fixed=1
+duration=9
+author=admin
+comment=Synchronised downtime
+}
+servicedowntime {
+host_name=host1
+service_description=Dummy service2
+entry_time=1264627948
+start_time=2034567991
+end_time=2034567995
+fixed=1
+duration=9
+author=admin
+comment=Synchronised downtime
+}
+servicedowntime {
+host_name=opslave
+service_description=Check Loadavg
+entry_time=1262254781
+start_time=1293790764
+end_time=1293797964
+fixed=1
+duration=7200
+author=admin
+comment=Host group 'Monitoring Servers': test future downtime
+}

Property changes on: t-tap/smallconfig/sync.dat.in
___________________________________________________________________
Added: svn:mergeinfo

Index: t-tap/smallconfig/nagios.cfg
===================================================================
--- t-tap/smallconfig/nagios.cfg	(revision 1)
+++ t-tap/smallconfig/nagios.cfg	(working copy)
@@ -623,6 +623,7 @@
 # variable is set to 1.
 
 state_retention_file=smallconfig/retention.dat
+sync_retention_file=smallconfig/sync.dat
 
 
 
Index: t-tap/smallconfig/retention.dat
===================================================================
--- t-tap/smallconfig/retention.dat	(revision 1)
+++ t-tap/smallconfig/retention.dat	(working copy)
@@ -56,6 +56,60 @@
 plugin_output=PING CRITICAL - Packet loss = 100%
 long_plugin_output=
 performance_data=rta=5000.000000ms;3000.000000;5000.000000;0.000000 pl=100%;80;100;0
+last_check=1111111111
+next_check=1264456444
+check_options=0
+current_attempt=1
+max_attempts=2
+normal_check_interval=0.000000
+retry_check_interval=0.000000
+state_type=1
+last_state_change=1264456444
+last_hard_state_change=1264456444
+last_time_up=1247509815
+last_time_down=1264456444
+last_time_unreachable=1264456432
+notified_on_down=1
+notified_on_unreachable=1
+last_notification=0
+current_notification_number=52
+current_notification_id=13392
+notifications_enabled=1
+problem_has_been_acknowledged=1
+acknowledgement_type=0
+active_checks_enabled=1
+passive_checks_enabled=1
+event_handler_enabled=0
+flap_detection_enabled=1
+failure_prediction_enabled=1
+process_performance_data=1
+obsess_over_host=0
+is_flapping=1
+percent_state_change=29.87
+check_flapping_recovery_notification=1
+state_history=1,1,1,1,2,2,1,2,2,1,1,1,1,1,1,1,2,2,2,2,1
+}
+host {
+host_name=hostveryrecent
+modified_attributes=1
+check_command=check_host!-H $HOSTADDRESS$ -w 3000.0,80% -c 5000.0,100% -p 1
+check_period=24x7
+notification_period=24x7
+event_handler=
+has_been_checked=1
+check_execution_time=10.030
+check_latency=0.000
+check_type=0
+current_state=1
+last_state=2
+last_hard_state=2
+last_event_id=1204
+current_event_id=1275
+current_problem_id=41
+last_problem_id=0
+plugin_output=PING CRITICAL - Packet loss = 100%
+long_plugin_output=
+performance_data=rta=5000.000000ms;3000.000000;5000.000000;0.000000 pl=100%;80;100;0
 last_check=1264456433
 next_check=1264456444
 check_options=0
@@ -168,3 +222,27 @@
 author=(Nagios Process)
 comment_data=Notifications for this service are being suppressed because it was detected as having been flapping between different states (40.3% change >= 20.0% threshold).  When the service state stabilizes and the flapping stops, notifications will be re-enabled.
 }
+hostdowntime {
+host_name=host1
+downtime_id=19
+entry_time=1264627948
+start_time=2034567890
+end_time=2034567899
+triggered_by=0
+fixed=1
+duration=9
+author=admin
+comment=Downtime set way into future
+}
+hostdowntime {
+host_name=host1
+downtime_id=20
+entry_time=1264627948
+start_time=2034567891
+end_time=2034567900
+triggered_by=0
+fixed=1
+duration=9
+author=admin
+comment=Overlapping future downtime
+}
Index: common/comments.c
===================================================================
--- common/comments.c	(revision 1)
+++ common/comments.c	(working copy)
@@ -706,14 +706,29 @@
 	comment *temp_comment=NULL;
 
 	for(temp_comment=comment_list;temp_comment!=NULL;temp_comment=temp_comment->next){
-		if(temp_comment->comment_id==comment_id && temp_comment->comment_type==comment_type)
+        if(comment_type!=ANY_COMMENT && temp_comment->comment_type!=comment_type)
+            continue;
+		if(temp_comment->comment_id==comment_id)
 			return temp_comment;
 	        }
 
 	return NULL;
         }
 
+/* find a comment by comment_type, host_name, service_desc (NULL if hostcomment), entry_time, author, comment_data */
+comment *find_comment_by_similar_content(int comment_type, char *hostname, char *service_description, char *author, char *comment_data){
+    comment *temp_comment=NULL;
 
+    for(temp_comment=comment_list;temp_comment!=NULL;temp_comment=temp_comment->next){
+        if(temp_comment->comment_type==comment_type 
+            && strcmp(temp_comment->host_name,hostname)==0 
+            && (service_description==NULL || strcmp(temp_comment->service_description,service_description)==0) 
+            && strcmp(temp_comment->author,author)==0 
+            && strcmp(temp_comment->comment_data,comment_data)==0)
+            return temp_comment;
+            }
+    return NULL;
+    }
 
 
 
Index: common/downtime.c
===================================================================
--- common/downtime.c	(revision 1)
+++ common/downtime.c	(working copy)
@@ -1045,7 +1045,36 @@
         }
 
 
+/* finds a specific downtime entry by similar content (in a distributed environment, downtime_ids are not synchronised) */
+scheduled_downtime *find_downtime_by_similar_content(int type, char *host_name, char *service_description, char *author, char *comment_data, time_t start_time, time_t end_time, int fixed, unsigned long duration){
+	scheduled_downtime *temp_downtime=NULL;
 
+    if(service_description==NULL) {
+        if(type==ANY_DOWNTIME)
+            type=HOST_DOWNTIME;
+        /* Must specify a service_description if you are searching services - obviously! */
+        if(type==SERVICE_DOWNTIME)
+            return NULL;
+        }
+
+	for(temp_downtime=scheduled_downtime_list;temp_downtime!=NULL;temp_downtime=temp_downtime->next){
+		if(type!=ANY_DOWNTIME && temp_downtime->type!=type)
+			continue;
+		if(temp_downtime->start_time==start_time 
+            && temp_downtime->end_time==end_time 
+            && temp_downtime->fixed==fixed
+            && temp_downtime->duration==duration
+            && strcmp(temp_downtime->host_name,host_name)==0 
+            && (service_description==NULL || (temp_downtime->type == SERVICE_DOWNTIME && strcmp(temp_downtime->service_description,service_description)==0)) 
+            && strcmp(temp_downtime->author,author)==0 
+            && strcmp(temp_downtime->comment,comment_data)==0 
+            )
+			return temp_downtime;
+	        }
+
+	return NULL;
+        }
+
 /******************************************************************/
 /********************* CLEANUP FUNCTIONS **************************/
 /******************************************************************/
Index: base/commands.c
===================================================================
--- base/commands.c	(revision 1)
+++ base/commands.c	(working copy)
@@ -271,6 +271,8 @@
 		command_type=CMD_SAVE_STATE_INFORMATION;
 	else if(!strcmp(command_id,"READ_STATE_INFORMATION"))
 		command_type=CMD_READ_STATE_INFORMATION;
+	else if(!strcmp(command_id,"SYNC_STATE_INFORMATION"))
+		command_type=CMD_SYNC_STATE_INFORMATION;
 
 	else if(!strcmp(command_id,"ENABLE_EVENT_HANDLERS"))
 		command_type=CMD_ENABLE_EVENT_HANDLERS;
@@ -782,6 +784,10 @@
 		read_initial_state_information();
 		break;
 
+    case CMD_SYNC_STATE_INFORMATION:
+        sync_state_information();
+        break;
+
 	case CMD_ENABLE_NOTIFICATIONS:
 		enable_all_notifications();
 		break;
Index: base/config.c
===================================================================
--- base/config.c	(revision 1)
+++ base/config.c	(working copy)
@@ -1329,6 +1329,8 @@
 			continue;
 		else if(strstr(input,"state_retention_file=")==input)
 			continue;
+		else if(strstr(input,"sync_retention_file=")==input)
+			continue;
 		else if(strstr(input,"object_cache_file=")==input)
 			continue;
 		else if(strstr(input,"precached_object_file=")==input)
Index: base/nagios.c
===================================================================
--- base/nagios.c	(revision 1)
+++ base/nagios.c	(working copy)
@@ -805,6 +805,7 @@
 			/* read initial service and host state information  */
 			initialize_retention_data(config_file);
 			read_initial_state_information();
+			sync_state_information();
 
 			/* initialize comment data */
 			initialize_comment_data(config_file);
Index: base/sretention.c
===================================================================
--- base/sretention.c	(revision 1)
+++ base/sretention.c	(working copy)
@@ -142,4 +142,22 @@
         }
 
 
+/* syncs host and state information from sync file */
+/* Should this go within read_state_information()? */
+int sync_state_information(void){
+	int result=OK;
 
+	if(retain_state_information==FALSE)
+		return OK;
+
+	/********* IMPLEMENTATION-SPECIFIC INPUT FUNCTION ********/
+#ifdef USE_XRDDEFAULT
+	result=xrddefault_sync_state_information();
+#endif
+
+	if(result==ERROR)
+		return ERROR;
+
+	return OK;
+        }
+
